@@ -4,6 +4,7 @@
 import numpy as np
 from bert4keras.layers import *
 from bert4keras.snippets import delete_arguments
+from bert4keras.snippets import is_string
 from keras.models import Model
 import json
 
@@ -270,6 +271,8 @@ class BERT(Transformer):
         self.with_nsp = with_nsp
         self.with_mlm = with_mlm
         self.custom_position_ids = custom_position_ids
+        if self.with_nsp and not self.with_pool:
+            self.with_pool = True
 
     def get_inputs(self):
         """BERT的输入是token_ids和segment_ids
@@ -432,7 +435,7 @@ class BERT(Transformer):
         z = self.layer_norm_conds[0]
         outputs = [x]
 
-        if self.with_pool or self.with_nsp:
+        if self.with_pool:
             # Pooler部分（提取CLS向量）
             x = outputs[0]
             x = self.apply(
@@ -1895,8 +1898,6 @@ def build_transformer_model(
     if 'dropout_rate' not in configs:
         configs['dropout_rate'] = configs.get('hidden_dropout_prob')
 
-    model, application = model.lower(), application.lower()
-
     models = {
         'bert': BERT,
         'albert': ALBERT,
@@ -1906,8 +1907,14 @@ def build_transformer_model(
         'gpt2_ml': GPT2_ML,
         't5': T5,
     }
-    MODEL = models[model]
 
+    if is_string(model):
+        model = model.lower()
+        MODEL = models[model]
+    else:
+        MODEL = model
+
+    application = application.lower()
     if model != 't5':
         if application == 'lm':
             MODEL = extend_with_language_model(MODEL)
