@@ -6,6 +6,7 @@ import logging
 import numpy as np
 import re
 import sys
+from collections import defaultdict
 
 _open_ = open
 is_py2 = six.PY2
@@ -110,35 +111,6 @@ class open:
         self.close()
 
 
-class Progress:
-    """显示进度，自己简单封装，比tqdm更可控一些
-    iterable: 可迭代的对象；
-    period: 显示进度的周期；
-    steps: iterable可迭代的总步数，相当于len(iterable)
-    """
-    def __init__(self, iterable, period=1, steps=None, desc=None):
-        self.iterable = iterable
-        self.period = period
-        if hasattr(iterable, '__len__'):
-            self.steps = len(iterable)
-        else:
-            self.steps = steps
-        self.desc = desc
-        if self.steps:
-            self._format_ = u'%s/%s passed' % ('%s', self.steps)
-        else:
-            self._format_ = u'%s passed'
-        if self.desc:
-            self._format_ = self.desc + ' - ' + self._format_
-        self.logger = logging.getLogger()
-
-    def __iter__(self):
-        for i, j in enumerate(self.iterable):
-            if (i + 1) % self.period == 0:
-                self.logger.info(self._format_ % (i + 1))
-            yield j
-
-
 def parallel_apply(
     func, iterable, workers, max_queue_size, callback=None, dummy=False
 ):
@@ -535,25 +507,47 @@ def delete_arguments(*arguments):
     return actual_decorator
 
 
-def groupby(iterable, key=None):
-    """类似itertools.groupby，但这里的key是iterable对象
+def longest_common_substring(source, target):
+    """最长公共子串（source和target的最长公共切片区间）
+    返回：子串长度, 所在区间（四元组）
+    注意：最长公共子串可能不止一个，所返回的区间只代表其中一个。
     """
-    if key is None:
-        key = iterable
+    c, l, span = defaultdict(int), 0, (0, 0, 0, 0)
+    for i, si in enumerate(source, 1):
+        for j, tj in enumerate(target, 1):
+            if si == tj:
+                c[i, j] = c[i - 1, j - 1] + 1
+                if c[i, j] > l:
+                    l = c[i, j]
+                    span = (i - l, i, j - l, j)
+    return l, span
 
-    result = []
-    for i, (k, v) in enumerate(zip(key, iterable)):
-        if i == 0:
-            result.append((k, [v]))
-            last_k = k
-        else:
-            if k == last_k:
-                result[-1][1].append(v)
+
+def longest_common_subsequence(source, target):
+    """最长公共子序列（source和target的最长非连续子序列）
+    返回：子序列长度, 映射关系（映射对组成的list）
+    注意：最长公共子序列可能不止一个，所返回的映射只代表其中一个。
+    """
+    c = defaultdict(int)
+    for i, si in enumerate(source, 1):
+        for j, tj in enumerate(target, 1):
+            if si == tj:
+                c[i, j] = c[i - 1, j - 1] + 1
+            elif c[i, j - 1] > c[i - 1, j]:
+                c[i, j] = c[i, j - 1]
             else:
-                result.append((k, [v]))
-                last_k = k
-
-    return result
+                c[i, j] = c[i - 1, j]
+    l, mapping = c[len(source), len(target)], []
+    i, j = len(source) - 1, len(target) - 1
+    while len(mapping) < l:
+        if source[i] == target[j]:
+            mapping.append((i, j))
+            i, j = i - 1, j - 1
+        elif c[i + 1, j] > c[i, j + 1]:
+            j = j - 1
+        else:
+            i = i - 1
+    return l, mapping[::-1]
 
 
 class Hook:
